//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __InjRea__
#define __InjRea__

#include "Chain.h"
#include "TH1.h"
#include "TMath.h"

using namespace std;

/**
 * Read a set of injections.
 * Injection parameters, generated with the InjGen tools, are loaded in a TTree structure. Injections are indexed and parameters can be loaded. Note that this class was optimized (in terms of speed) to load injections incrementally.
 * \author    Florent Robinet
 */
class InjRea {

  friend class InjRec;///< used to clone InjTree

 public:
  
  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the InjRea class.
   * Injections in the ROOT files designated by a file pattern are loaded. Each injection is tagged to 'true' by default.
   * It is important that the injection files were generated with the InjGen class. Injection files must be time-sorted and with no overlap.
   * @param aPattern injection file pattern
   * @param aVerbose verbosity level
   */
  InjRea(const string aPattern, const int aVerbose=0);

  /**
   * Destructor of the InjRea class.
   */
  virtual ~InjRea();
  /**
     @}
  */
  
  /**
   * Loads a given injection.
   * @param aInjIndex injection index
   */
  inline bool LoadInjection(const int aInjIndex){if(InjTree->GetEntry(aInjIndex)>0){ InjIndex=aInjIndex; return true; } return false; };

  /**
   * Returns the SineGauss h+rss peak amplitude of current injection.
   * 
   */
  double GetSineGaussh0plus(void);

  /**
   * Returns the SineGauss hxrss peak amplitude of current injection.
   * 
   */
  double GetSineGaussh0cross(void);

  /**
   * Returns the injection time.
   */
  inline double GetInjectionTime(void){ return inj_time; };

  /**
   * Returns the injection right-ascension.
   */
  inline double GetInjectionRa(void){ return inj_ra; };

  /**
   * Returns the injection declination.
   */
  inline double GetInjectionDec(void){ return inj_dec; };

  /**
   * Returns the injection eccentricity.
   */
  inline double GetInjectionEccentricity(void){ return inj_ecc; };

  /**
   * Returns the injection polarization.
   */
  inline double GetInjectionPolarization(void){ return inj_psi; };

  /**
   * Returns the injection amplitude.
   */
  inline double GetInjectionAmplitude(void){ return inj_amp; };

  /**
   * Returns the minimum injection amplitude.
   */
  inline double GetInjectionAmplitudeMin(void){ return inj_ampmin; };

  /**
   * Returns the maximum injection amplitude.
   */
  inline double GetInjectionAmplitudeMax(void){ return inj_ampmax; };
  
  /**
   * Returns the injection sigma.
   */
  inline double GetInjectionSigma(void){ return inj_sigma; };

  /**
   * Returns the injection frequency.
   */
  inline double GetInjectionFrequency(void){ return inj_f0; };

  /**
   * Returns the minimum injection frequency.
   */
  inline double GetInjectionFrequencyMin(void){ return inj_f0min; };

  /**
   * Returns the maximum injection frequency.
   */
  inline double GetInjectionFrequencyMax(void){ return inj_f0max; };

  /**
   * Returns the injection tag.
   */
  inline bool GetInjectionTag(void){ return inj_tag[InjIndex]; };

  /**
   * Sets a new tag value for the current injection.
   */
  inline void SetInjectionTag(const bool aNewTag){ inj_tag[InjIndex]=aNewTag; };

  /**
   * Returns the first injection index included in a given time range.
   * Returns -1 if no injection is found in this time range.
   * @param aTimeStart GPS range start
   * @param aTimeEnd GPS range stop
   */
  int GetFirstInjectionIndex(const double aTimeStart, const double aTimeEnd);
  
  /**
   * Returns the last injection index included in a given time range.
   * Returns -1 if no injection is found in this time range.
   * @param aTimeStart GPS range start
   * @param aTimeEnd GPS range stop
   */
  int GetLastInjectionIndex(const double aTimeStart, const double aTimeEnd);
  
  /**
   * Returns the class status.
   */
  inline bool GetStatus(void){ return status_OK; };
  
  /**
   * Returns the number of injections.
   */
  inline int GetNInjections(void){ return InjTree->GetEntries(); };
  
  /**
   * Returns the injection type.
   */
  inline string GetInjectionType(void){ return (string)InjTree->GetName(); };
 
  /**
   * Returns the maximal value for sigma.
   */
  inline double GetInjectionSigmaMax(void){ return inj_sigmamax; };

  /**
   * Returns the 1D distribution of an injection parameter.
   * The returned histograms must be deleted by the user.
   * NB: only injections tagged to true are considered.
   * @param aParamName parameter name (currently supported: "amplitude" and "f0")
   * @param aNbins number of bins (must be >0)
   * @param aBinType binning type (currently supported: "UNIFORM" and "LOG")
   */
  TH1D* GetInjectionParamDist(const string aParamName, const int aNbins=1, const string aBinType="UNIFORM");

  /**
   * Returns the input file pattern provided in the constructor.
   */
  string GetInputFilePattern(void){ return InjTree->GetInputFilePattern(); };
 
 protected:
  
  bool status_OK;            ///< status of the InjRea object
  int Verbose;               ///< verbosity level

  //inline int GetTreeNumber(void){ return InjTree->GetTreeNumber(); };
  inline Long64_t* GetTreeOffset(void){ return InjTree->GetTreeOffset(); };

  double inj_ra,             ///< injection right ascension
    inj_dec,                 ///< injection declination
    inj_psi,                 ///< injection polarization angle
    inj_time,                ///< injection time
    inj_amp,                 ///< injection amplitude
    inj_f0,                  ///< injection frequency
    inj_sigma,               ///< injection sigma
    inj_ecc;                 ///< injection eccentricity

  double inj_sigmamin;       ///< sigma min
  double inj_sigmamax;       ///< sigma max
  double inj_ampmin;         ///< amplitude min
  double inj_ampmax;         ///< amplitude max
  double inj_f0min;          ///< frequency min
  double inj_f0max;          ///< frequency max
 
 private:

  Chain *InjTree;            ///< injection tree
  bool *inj_tag;             ///< injection tags
  int InjIndex;              ///< current injection index
  double inj_timefirst;      ///< time of the first injection
  double inj_timelast;       ///< time of the last injection
 
  ClassDef(InjRea,0)
};

#endif


