//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "TriggerSelect.h"

ClassImp(TriggerSelect)

////////////////////////////////////////////////////////////////////////////////////
TriggerSelect::TriggerSelect(const int aVerbose){ 
////////////////////////////////////////////////////////////////////////////////////
  Verbose=aVerbose;

  // default time
  tmin_0   = 700000000.0;
  tmax_0   = 2000000000.0;
  ntbins_0 = 1;
  tbins = new double [0];
 
  // default frequency
  fmin_0   = 1.0;
  fmax_0   = 10000.0;
  nfbins_0 = 1;
  fbins = new double [0];

  // default Q
  qmin_0   = 1.0;
  qmax_0   = 1000.0;
  nqbins_0 = 1;
  qbins = new double [0];
 
  // default SNR
  smin_0   = 1.0;
  smax_0   = 1e20;
  nsbins_0 = 1;
  sbins = new double [0];
 
  // set default selection
  ResetSelection();
}

////////////////////////////////////////////////////////////////////////////////////
TriggerSelect::TriggerSelect(ReadTriggers *aTriggers, const int aVerbose){ 
////////////////////////////////////////////////////////////////////////////////////
  Verbose=aVerbose;

  // default time
  tmin_0   = aTriggers->GetTimeMin(); if(tmin_0==1e20) tmin_0=700000000.0;
  tmax_0   = aTriggers->GetTimeMax(); if(tmax_0==-1.0) tmax_0=2000000000.0;
  ntbins_0 = 1;
  tbins = new double [0];
 
  // default frequency
  fmin_0   = aTriggers->GetFrequencyMin(); if(fmin_0==1e20) fmin_0=1.0;
  fmax_0   = aTriggers->GetFrequencyMax(); if(fmax_0==-1.0) fmax_0=10000.0;
  nfbins_0 = 1;
  fbins = new double [0];

  // default Q
  qmin_0   = aTriggers->GetQMin();      if(qmin_0==1e20)  qmin_0=1.0;
  qmax_0   = 1.01*aTriggers->GetQMax(); if(qmax_0==-1.01) qmax_0=1000.0;
  nqbins_0 = 1;
  qbins = new double [0];
 
  // default SNR
  smin_0   = aTriggers->GetSNRMin();      if(smin_0==1e20)  smin_0=1.0;
  smax_0   = 1.01*aTriggers->GetSNRMax(); if(smax_0==-1.01) smax_0=1e20;
  nsbins_0 = 1;
  sbins = new double [0];
 
  // set default selection
  ResetSelection();
}

////////////////////////////////////////////////////////////////////////////////////
TriggerSelect::~TriggerSelect(void){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"TriggerSelect::~TriggerSelect"<<endl;
  delete tbins;
  delete fbins;
  delete qbins;
  delete sbins;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::ResetSelection(void){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose) cout<<"TriggerSelect::ResetSelection: set default selection"<<endl;

  // time
  tmin   = tmin_0;
  tmax   = tmax_0;
  ntbins = ntbins_0;
  MakeTbins();

  // frequency
  fmin   = fmin_0;
  fmax   = fmax_0;
  nfbins = nfbins_0;
  MakeFbins();

  // Q
  qmin   = qmin_0;
  qmax   = qmax_0;
  nqbins = nqbins_0;
  MakeQbins();

  // SNR
  smin   = smin_0;
  smax   = smax_0;
  nsbins = nsbins_0;
  MakeSbins();

  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetTimeSelection(const int aResolution, const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  tmin=aMin;
  tmax=aMax;
  ntbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetTimeSelection: set time selection = "<<tmin<<"-"<<tmax<<", resolution = "<<ntbins<<endl;
  return MakeTbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetTimeResolution(const int aResolution){
////////////////////////////////////////////////////////////////////////////////////
  ntbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetTimeResolution: set time resolution = "<<ntbins<<endl;
  return MakeTbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetTimeMin(const double aMin){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"TriggerSelect::SetTimeMin: set minimal time = "<<aMin<<endl;
  tmin=fabs(aMin);
  return MakeTbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetTimeMax(const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"TriggerSelect::SetTimeMax: set maximal time = "<<aMax<<endl;
  tmax=fabs(aMax);
  return MakeTbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetTimeRange(const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  if(Verbose>1) cout<<"TriggerSelect::SetTimeRange: set time range = "<<aMin<<"-"<<aMax<<endl;
  tmin=aMin;
  tmax=aMax;
  return MakeTbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::MakeTbins(void){
////////////////////////////////////////////////////////////////////////////////////
  delete tbins;
  tbins = new double [ntbins+1];
  for(int b=0; b<ntbins+1; b++) 
    tbins[b] = tmin + b * (tmax-tmin)/(double)ntbins;
  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetFrequencySelection(const int aResolution, const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  fmin=fabs(aMin);
  fmax=fabs(aMax);
  nfbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetFrequencySelection: set frequency selection = "<<fmin<<"-"<<fmax<<", resolution = "<<nfbins<<endl;
  return MakeFbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetFrequencyResolution(const int aResolution){
////////////////////////////////////////////////////////////////////////////////////
  nfbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetFrequencyResolution: set frequency resolution = "<<nfbins<<endl;
  return MakeFbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetFrequencyMin(const double aMin){
////////////////////////////////////////////////////////////////////////////////////
  fmin=fabs(aMin);
  if(Verbose>1) cout<<"TriggerSelect::SetFrequencyMin: set minimal frequency = "<<fmin<<endl;
  return MakeFbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetFrequencyMax(const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  fmax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetFrequencyMax: set maximal frequency = "<<fmax<<endl;
  return MakeFbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetFrequencyRange(const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  fmin=fabs(aMin);
  fmax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetFrequencyRange: set frequency range = "<<fmin<<"-"<<fmax<<endl;
  return MakeFbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::MakeFbins(void){
////////////////////////////////////////////////////////////////////////////////////
  delete fbins;
  fbins = new double [nfbins+1];
  for(int b=0; b<nfbins+1; b++) 
    fbins[b] = fmin*pow(10.0,b*log10(fmax/fmin)/(double)nfbins);
  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetQSelection(const int aResolution, const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  qmin=fabs(aMin);
  qmax=fabs(aMax);
  nqbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetQSelection: set Q selection = "<<qmin<<"-"<<qmax<<", resolution = "<<nqbins<<endl;
  return MakeQbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetQResolution(const int aResolution){
////////////////////////////////////////////////////////////////////////////////////
  nqbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetQResolution: set Q resolution = "<<nqbins<<endl;
  return MakeQbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetQMin(const double aMin){
////////////////////////////////////////////////////////////////////////////////////
  qmin=fabs(aMin);
  if(Verbose>1) cout<<"TriggerSelect::SetQMin: set minimal Q = "<<qmin<<endl;
  return MakeQbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetQMax(const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  qmax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetQMax: set maximal Q = "<<qmax<<endl;
  return MakeQbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetQRange(const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  qmin=fabs(aMin);
  qmax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetQRange: set Q range = "<<qmin<<"-"<<qmax<<endl;
  return MakeQbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::MakeQbins(void){
////////////////////////////////////////////////////////////////////////////////////
  delete qbins;
  qbins = new double [nqbins+1];
  for(int b=0; b<nqbins+1; b++) 
    qbins[b] = qmin*pow(10.0,b*log10(qmax/qmin)/(double)nqbins);
  return;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetSNRSelection(const int aResolution, const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  smin=fabs(aMin);
  smax=fabs(aMax);
  nsbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetSNRSelection: set SNR selection = "<<smin<<"-"<<smax<<", resolution = "<<nsbins<<endl;
  return MakeSbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetSNRResolution(const int aResolution){
////////////////////////////////////////////////////////////////////////////////////
  nsbins=(int)fabs(aResolution);
  if(Verbose>1) cout<<"TriggerSelect::SetSNRResolution: set SNR resolution = "<<nsbins<<endl;
  return MakeSbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetSNRMin(const double aMin){
////////////////////////////////////////////////////////////////////////////////////
  smin=fabs(aMin);
  if(Verbose>1) cout<<"TriggerSelect::SetSNRMin: set minimal SNR = "<<smin<<endl;
  return MakeSbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetSNRMax(const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  smax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetSNRMax: set maximal SNR = "<<smax<<endl;
  return MakeSbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::SetSNRRange(const double aMin, const double aMax){
////////////////////////////////////////////////////////////////////////////////////
  smin=fabs(aMin);
  smax=fabs(aMax);
  if(Verbose>1) cout<<"TriggerSelect::SetSNRRange: set SNR range = "<<smin<<"-"<<smax<<endl;
  return MakeSbins();
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::MakeSbins(void){
////////////////////////////////////////////////////////////////////////////////////
  delete sbins;
  sbins = new double [nsbins+1];
  for(int b=0; b<nsbins+1; b++) 
    sbins[b] = smin*pow(10.0,b*log10(smax/smin)/(double)nsbins);
  return;
}

////////////////////////////////////////////////////////////////////////////////////
TH1D* TriggerSelect::GetTH1D(const string aParameter){
////////////////////////////////////////////////////////////////////////////////////
  if(!aParameter.compare("Time")&&tmin<tmax)
     return new TH1D(aParameter.c_str(),aParameter.c_str(),ntbins,tbins);
  else if(!aParameter.compare("Frequency")&&fmin<fmax)
     return new TH1D(aParameter.c_str(),aParameter.c_str(),nfbins,fbins);
  else if(!aParameter.compare("Q")&&qmin<qmax)
     return new TH1D(aParameter.c_str(),aParameter.c_str(),nqbins,qbins);
  else if(!aParameter.compare("SNR")&&smin<smax)
     return new TH1D(aParameter.c_str(),aParameter.c_str(),nsbins,sbins);
  else;
  return NULL;
}

////////////////////////////////////////////////////////////////////////////////////
TH2D* TriggerSelect::GetTH2D(const string aParameterX, const string aParameterY){
////////////////////////////////////////////////////////////////////////////////////
  if(!aParameterX.compare("Time")&&tmin<tmax){
    if(!aParameterY.compare("Time")&&tmin<tmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),ntbins,tbins,ntbins,tbins);
    else if(!aParameterY.compare("Frequency")&&fmin<fmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),ntbins,tbins,nfbins,fbins);
    else if(!aParameterY.compare("Q")&&qmin<qmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),ntbins,tbins,nqbins,qbins);
    else if(!aParameterY.compare("SNR")&&smin<smax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),ntbins,tbins,nsbins,sbins);
    else;
  }
  else if(!aParameterX.compare("Frequency")&&fmin<fmax){
    if(!aParameterY.compare("Time")&&tmin<tmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nfbins,fbins,ntbins,tbins);
    else if(!aParameterY.compare("Frequency")&&fmin<fmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nfbins,fbins,nfbins,fbins);
    else if(!aParameterY.compare("Q")&&qmin<qmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nfbins,fbins,nqbins,qbins);
    else if(!aParameterY.compare("SNR")&&smin<smax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nfbins,fbins,nsbins,sbins);
    else;
  }
  else if(!aParameterX.compare("Q")&&qmin<qmax){
    if(!aParameterY.compare("Time")&&tmin<tmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nqbins,qbins,ntbins,tbins);
    else if(!aParameterY.compare("Frequency")&&fmin<fmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nqbins,qbins,nfbins,fbins);
    else if(!aParameterY.compare("Q")&&qmin<qmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nqbins,qbins,nqbins,qbins);
    else if(!aParameterY.compare("SNR")&&smin<smax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nqbins,qbins,nsbins,sbins);
    else;
  }
  else if(!aParameterX.compare("SNR")&&smin<smax){
    if(!aParameterY.compare("Time")&&tmin<tmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nsbins,sbins,ntbins,tbins);
    else if(!aParameterY.compare("Frequency")&&fmin<fmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nsbins,sbins,nfbins,fbins);
    else if(!aParameterY.compare("Q")&&qmin<qmax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nsbins,sbins,nqbins,qbins);
    else if(!aParameterY.compare("SNR")&&smin<smax)
      return new TH2D((aParameterY+aParameterX).c_str(),(aParameterY+aParameterX).c_str(),nsbins,sbins,nsbins,sbins);
    else;

  }
  else;
  return NULL;
}

////////////////////////////////////////////////////////////////////////////////////
void TriggerSelect::PrintSelection(void){
////////////////////////////////////////////////////////////////////////////////////
  cout<<"TriggerSelect::PrintSelection:"<<endl;
  cout<<"             Time:      "<<tmin<<" --> "<<tmax<<" ("<<ntbins<<" bins)"<<endl;
  cout<<"             Frequency: "<<fmin<<" --> "<<fmax<<" ("<<nfbins<<" bins)"<<endl;
  cout<<"             Q:         "<<qmin<<" --> "<<qmax<<" ("<<nqbins<<" bins)"<<endl;
  cout<<"             SNR:       "<<smin<<" --> "<<smax<<" ("<<nsbins<<" bins)"<<endl;
  return;
}
